<?php

namespace Drupal\ipless\Asset;

use Drupal\Core\Extension\ThemeHandlerInterface;
use Drupal\Core\Asset\LibraryDiscoveryInterface;
use Drupal\Core\Theme\ThemeManagerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Asset\AttachedAssets;
use Less_Parser;
use Drupal\Core\Theme\ActiveTheme;

/**
 * Description of AssetRenderer
 */
class AssetRenderer {

  /**
   * Less Preprocessor.
   * 
   * @var Less_Parser
   */
  protected $less;

  /**
   * Theme Handler.
   * 
   * @var \Drupal\Core\Extension\ThemeHandlerInterface
   */
  protected $themeHandler;

  /**
   * Library Discovery
   * 
   * @var \Drupal\Core\Asset\LibraryDiscoveryInterface
   */
  protected $libraryDiscovery;

  /**
   * Theme manager.
   * 
   * @var \Drupal\Core\Theme\ThemeManagerInterface
   */
  protected $themeManager;

  /**
   * Config Factory.
   * 
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Less Asset Resolver.
   * 
   * @var \Drupal\ipless\Asset\AssetResolverInterface
   */
  protected $assetResolver;

  public function __construct(ThemeHandlerInterface $theme_handler, LibraryDiscoveryInterface $library_discovery, ThemeManagerInterface $theme_manager, ConfigFactoryInterface $config_factory, AssetResolverInterface $asset_resolver) {
    $this->themeHandler     = $theme_handler;
    $this->libraryDiscovery = $library_discovery;
    $this->themeManager     = $theme_manager;
    $this->configFactory    = $config_factory;
    $this->assetResolver    = $asset_resolver;
  }

  /**
   * Render methode.
   */
  public function render($forced = FALSE) {
    $theme                         = $this->getTheme($forced);
    if (!$theme)
      return;

    $build['#attached']['library'] = $theme->getLibraries();
    $assets                        = AttachedAssets::createFromRenderArray($build);

    $less_assets = $this->assetResolver->getLessAssets($assets);

    foreach ($less_assets as $asset_options) {
      $this->compile($asset_options['data'], $asset_options['output']);
    }
  }

  /**
   * Get Theme.
   * 
   * @return bool|theme
   */
  protected function getTheme($forced) {
    if ($forced) {
      $this->theme = $this->getDefaultTheme();
      return $this->theme;
    }

    if (!empty($this->theme)) {
      return $this->theme;
    }
    else {
      $current_theme      = $this->themeManager->getActiveTheme();
      $config             = $this->configFactory->get('system.theme');
      $default_theme_name = $config->get('default');
      if ($current_theme->getName() != $default_theme_name)
        return FALSE;
      $this->theme        = $current_theme;
    }
    return $this->theme;
  }

  /**
   * Get Default Theme.
   * 
   * @return ActiveTheme
   */
  protected function getDefaultTheme() {
    $defaultTheme = $this->themeHandler->getDefault();

    $activeTheme = $this->themeHandler->getTheme($defaultTheme);

    if (empty($activeTheme))
      return FALSE;

    $data         = (array) $activeTheme;
    $data['name'] = $activeTheme->getName();

    return new ActiveTheme($data);
  }

  /**
   * Return Less processor.
   * 
   * @return Less_Parser
   */
  protected function getLess() {
    if (!$this->less) {
      $config = $this->configFactory->get('system.performance');

      $options    = ['sourceMap' => (bool) $config->get('ipless_sourcemap')];
      $this->less = new Less_Parser($options);
    }
    return $this->less;
  }

  /**
   * Compile Less file.
   */
  protected function compile($file, $output) {
    $less = $this->getLess();

    if ($less) {
      $less->reset();
      $less->parseFile($file);

      $this->preparePath($output);
      file_put_contents($output, $less->getCss());
    }
  }

  /**
   * Create path if not exist.
   */
  protected function preparePath($path) {
    $info = pathinfo($path);

    if (!empty($info['dirname']) && !file_exists($path)) {
      file_prepare_directory($info['dirname'], FILE_CREATE_DIRECTORY);
    }
  }

}
